@@ -28,6 +28,8 @@ from flask import session, jsonify, Blueprint, make_response, abort, \
     redirect, url_for, after_this_request, send_file, send_from_directory, \
     safe_join, escape, Markup, render_template, current_app, g
 
+from werkzeug.datastructures import CombinedMultiDict, MultiDict
+
 import logging
 log = logging.getLogger("fsa")
 
@@ -57,7 +59,7 @@ def bool_cast(s: str) -> Optional[bool]:
 
 def int_cast(s: str) -> Optional[int]:
     """Parses an integer, allowing several bases."""
-    return int(s, base=0) if s is not None else None
+    return int(s, base=0) if s else None
 
 
 class path(str):
@@ -66,7 +68,7 @@ class path(str):
 
 
 class string(str):
-    """Type to distinguish str path parameters."""
+    """Type to distinguish str string parameters."""
     pass
 
 
@@ -98,7 +100,7 @@ def typeof(p: inspect.Parameter):
         return list
     elif p.annotation is not inspect._empty:  # type: ignore
         return p.annotation
-    elif p.default is not None and p.default is not inspect._empty:  # type: ignore
+    elif p.default and p.default is not inspect._empty:  # type: ignore
         return type(p.default)  # type inference!
     else:
         return str
@@ -132,7 +134,7 @@ class Reference:
         # keep track of initial methods for later cleanup
         self._init: Set[str] = set()
         self._init.update(self.__dir__())
-        if obj is not None:
+        if obj:
             self._set_obj(obj)
 
     def _set_obj(self, obj):
@@ -186,14 +188,14 @@ class Reference:
 # - automatic reset based on cache efficiency? expansion?
 #
 class CacheOK:
-    """Positive caching decorator.
+    """Positive caching decorator for boolean functions.
 
     Cache True answers, but still forwards False answers to the underlying
     function.
     """
 
     def __init__(self, fun: Callable[[List[Any]], bool]):
-        self._fun = fun
+        self.__wrapped__ = fun
         self._cache: Set[Any] = set()
         self.cache_clear = self._cache.clear
 
@@ -201,7 +203,7 @@ class CacheOK:
         if args in self._cache:
             return True
         else:
-            ok = self._fun(*args)
+            ok = self.__wrapped__(*args)
             if ok:
                 self._cache.add(args)
             return ok
@@ -226,6 +228,11 @@ class Flask(flask.Flask):
         # overwritten late because called by upper Flask initialization for "static"
         self.add_url_rule = self._fsa.add_url_rule
 
+    def clear_caches(self):
+        """Clear all internal caches."""
+        self._fsa.clear_caches()
+
+    # hooks
     def get_user_pass(self, gup):
         """Set `get_user_pass` helper function."""
         return self._fsa.get_user_pass(gup)
@@ -234,6 +241,7 @@ class Flask(flask.Flask):
         """Set `user_in_group` helper function."""
         return self._fsa.user_in_group(uig)
 
+    # password management
     def check_password(self, pwd, ref):
         """Check whether password is ok wrt to current password manager."""
         return self._fsa.check_password(pwd, ref)
@@ -242,28 +250,24 @@ class Flask(flask.Flask):
         """Hash password using current password manager scheme."""
         return self._fsa.hash_password(pwd)
 
+    # token
     def create_token(self, user: str = None):
         """Create a token with the current token scheme."""
         return self._fsa.create_token(user)
 
+    # user
     def get_user(self):
-        """Authenticate remote user."""
+        """Authenticate remote user or raise exception."""
         return self._fsa.get_user()
 
     def current_user(self):
-        """Get current authenticated user, if any."""
+        """Get current authenticated user, if any, or None."""
         return self._fsa.current_user()
 
-    def clear_caches(self):
-        """Clear internal caches."""
-        self._fsa.clear_caches()
-
 
 # actual extension
 class FlaskSimpleAuth:
-    """Flask extension to implement authentication, authorization and parameter
-    management.
-    """
+    """Flask extension for authentication, authorization and parameters."""
 
     def __init__(self, app: flask.Flask = None):
         """Constructor parameter: flask application to extend."""
@@ -271,6 +275,8 @@ class FlaskSimpleAuth:
         self._maxsize = 1024
         self._get_user_pass = None
         self._user_in_group = None
+        self._auth: List[str] = []
+        self._saved_auth: Optional[List[str]] = None
         self._http_auth = None
         self._pm = None
         # actual main initialization is deferred
@@ -287,8 +293,9 @@ class FlaskSimpleAuth:
         """Before request hook to perform early authentication."""
         self._user = None
         self._need_authorization = True
-        if self._mode != "always":
+        if self._mode == "lazy":
             return
+        # keep on under always & all
         for skip in self._skip_path:
             if skip(request.path):
                 return
@@ -296,13 +303,14 @@ class FlaskSimpleAuth:
             self._user = self.get_user()
         except AuthException as e:
             return self._Resp(e.message, e.status)
-        assert self._user is not None
+        assert self._user
 
     def _auth_after_cleanup(self, res: Response):
         """After request hook to cleanup authentication and detect missing
         authorization."""
         self._user = None
         # should it always return 500?
+        # NOTE this may be to late to prevent a commit
         if res.status_code < 400 and self._need_authorization:
             method, path = request.method, request.path
             log.warning(f"missing authorization on {method} {path}")
@@ -312,10 +320,10 @@ class FlaskSimpleAuth:
 
     def _possible_redirect(self, res: Response):
         """After request hook to turn a 401 into a redirect."""
-        if res.status_code == 401 and self._401_redirect is not None:
+        if res.status_code == 401 and self._401_redirect:
             location = self._401_redirect
             # allow to come back later in some cases
-            if self._url_name is not None and request.method == "GET":
+            if self._url_name and request.method == "GET":
                 sep = "&" if "?" in self._url_name else "?"
                 import urllib
                 location += sep + urllib.parse.urlencode({self._url_name: request.url})
@@ -326,8 +334,8 @@ class FlaskSimpleAuth:
         """Set a cookie if needed and none was sent."""
         # NOTE thanks to max_age the client should not send stale cookies
         if self._carrier == "cookie":
-            assert self._token is not None and self._name is not None
-            if self._user is not None and self._can_create_token():
+            assert self._token and self._name
+            if self._user and self._can_create_token():
                 if self._name in request.cookies:
                     user, exp = self._get_any_token_auth_exp(request.cookies[self._name])
                     # reset token when only 25% time remains
@@ -354,23 +362,37 @@ class FlaskSimpleAuth:
             if self._auth_has("basic", "password"):
                 res.headers["WWW-Authenticate"] = f"Basic realm=\"{self._realm}\""
             elif self._auth_has("http-basic", "http-digest", "http-token", "digest"):
-                assert self._http_auth is not None
+                assert self._http_auth
                 res.headers["WWW-Authenticate"] = self._http_auth.authenticate_header()
             elif "token" in self._auth and self._carrier == "bearer":
                 res.headers["WWW-Authenticate"] = f"{self._name} realm=\"{self._realm}\""
             # else: scheme does not rely on WWW-Authenticate…
         # else: no need for WWW-Authenticate
+        # restore temporary auth if needed
+        if self._saved_auth:
+            self._auth, self._saved_auth = self._saved_auth, None
         return res
 
     def _cache_function(self, fun):
         """Generate or regenerate cache for function."""
-        # get the actual function when regenerating
-        if hasattr(fun, "__wrapped__"):
+        # get the actual function when regenerating caches
+        while hasattr(fun, "__wrapped__"):
             fun = fun.__wrapped__
         # NOTE probaly maxsize should disable with None and unbound with 0.
-        return fun if fun is None or self._maxsize == 0 else \
+        return fun if not fun or self._maxsize == 0 else \
             functools.lru_cache(maxsize=self._maxsize)(fun)
 
+    def _params(self):
+        """Get request parameters wherever they are."""
+        if request.json:
+            return request.json
+        else:
+            # reimplement "request.values" after Flask 2.0 regression
+            # https://github.com/pallets/werkzeug/pull/2037
+            # https://github.com/pallets/flask/issues/4120
+            return CombinedMultiDict([MultiDict(d) if not isinstance(d, MultiDict) else d
+                                      for d in (request.args, request.form)])
+
     def get_user_pass(self, gup):
         """Set `get_user_pass` helper, can be used as a decorator."""
         self._get_user_pass = self._cache_function(gup)
@@ -387,7 +409,7 @@ class FlaskSimpleAuth:
     #
     def initialize(self):
         """Run late initialization on current app."""
-        assert self._app is not None
+        assert self._app
         self.init_app(self._app)
 
     def init_app(self, app: flask.Flask):
@@ -397,15 +419,14 @@ class FlaskSimpleAuth:
         directives.
         """
         log.info("FSA initialization…")
-        assert app is not None
+        assert app
         self._app = app
         conf = app.config
         #
         # overall auth setup
         #
-        self._auth: List[str] = []
         auth = conf.get("FSA_AUTH", None)
-        if auth is None:
+        if not auth:
             self._auth = ["httpd"]
         elif isinstance(auth, str):
             if auth not in ("token", "http-token"):
@@ -437,7 +458,7 @@ class FlaskSimpleAuth:
         if self._carrier not in (None, "bearer", "param", "cookie", "header"):
             raise Exception(f"Unexpected token carrier (FSA_TOKEN_CARRIER): {self._carrier}")
         # sanity checks
-        if need_carrier and self._carrier is None:
+        if need_carrier and not self._carrier:
             raise Exception(f"Token type {self._token} requires a carrier")
         # name of token for cookie or param, Authentication scheme, or other header
         default_name: Optional[str] = None
@@ -448,7 +469,7 @@ class FlaskSimpleAuth:
         elif self._carrier == "header":
             default_name = "Auth"
         self._name = conf.get("FSA_TOKEN_NAME", default_name)
-        if need_carrier and self._name is None:
+        if need_carrier and not self._name:
             raise Exception(f"Token carrier {self._carrier} requires a name")
         # token realm…
         realm = conf.get("FSA_TOKEN_REALM", self._app.name)
@@ -464,7 +485,7 @@ class FlaskSimpleAuth:
         # token signature
         if "FSA_TOKEN_SECRET" in conf:
             self._secret = conf["FSA_TOKEN_SECRET"]
-            if self._secret is not None and len(self._secret) < 16:
+            if self._secret and len(self._secret) < 16:
                 log.warning("token secret is short")
         else:
             import random
@@ -473,7 +494,7 @@ class FlaskSimpleAuth:
             # list of 94 chars, about 6.5 bits per char
             chars = string.ascii_letters + string.digits + string.punctuation
             self._secret = "".join(random.SystemRandom().choices(chars, k=40))
-        if self._token is None:
+        if not self._token:
             pass
         elif self._token == "fsa":
             self._sign = self._secret
@@ -484,7 +505,7 @@ class FlaskSimpleAuth:
             self._algo = algo
             if algo[0] in ("R", "E", "P"):
                 self._sign = conf.get("FSA_TOKEN_SIGN", None)
-                if self._sign is None:
+                if not self._sign:
                     log.warning("cannot sign JWT token, assuming a third party provider")
             elif algo[0] == "H":
                 self._sign = self._secret
@@ -523,7 +544,7 @@ class FlaskSimpleAuth:
                 assert self._http_auth is not None  # for pleasing mypy
                 # FIXME? nonce & opaque callbacks? session??
             elif "http-token" in self._auth:
-                if self._carrier == "header" and "header" not in opts and self._name is not None:
+                if self._carrier == "header" and "header" not in opts and self._name:
                     opts["header"] = self._name
                 self._http_auth = fha.HTTPTokenAuth(scheme=self._name, realm=self._realm, **opts)
                 assert self._http_auth is not None  # for pleasing mypy
@@ -545,8 +566,21 @@ class FlaskSimpleAuth:
         # blueprint hacks
         #
         self.blueprints = self._app.blueprints
-        self._blueprint_order = self._app._blueprint_order
         self.debug = False
+        if hasattr(self._app, '_blueprint_order'):
+            # Flask 1.x
+            self._blueprint_order = self._app._blueprint_order
+        elif hasattr(self._app, '_is_setup_finished'):
+            # Flask 2.0
+            self._is_setup_finished = self._app._is_setup_finished
+            self.before_request_funcs = self._app.before_request_funcs
+            self.after_request_funcs = self._app.after_request_funcs
+            self.teardown_request_funcs = self._app.teardown_request_funcs
+            self.url_default_functions = self._app.url_default_functions
+            self.url_value_preprocessors = self._app.url_value_preprocessors
+            self.template_context_processors = self._app.template_context_processors
+        else:
+            log.warning("unexpected Flask version while dealing with blueprints?")
         #
         # caches
         #
@@ -559,13 +593,13 @@ class FlaskSimpleAuth:
         """Deferred password manager initialization."""
         # only initialize if some password may need to be checked
         # so that passlib is not imported for nothing
-        if self._get_user_pass is None or self._pm is not None:
+        if not self._get_user_pass or self._pm:
             return
-        assert self._app is not None
+        assert self._app
         conf = self._app.config
         scheme = conf.get("FSA_PASSWORD_SCHEME", "bcrypt")
         log.info(f"initializing password manager with {scheme}")
-        if scheme is not None:
+        if scheme:
             if scheme == "plaintext":
                 log.warning("plaintext password manager is a bad idea")
             # passlib context is a pain, you have to know the scheme name to set its
@@ -595,10 +629,10 @@ class FlaskSimpleAuth:
         assert request.remote_user is None, "do not shadow web server auth"
         assert request.environ["REMOTE_ADDR"][:4] == "127.", \
             "fake auth only on localhost"
-        params = request.json or request.values
+        params = self._params()
         user = params.get(self._login, None)
         # it could check that the user exists in db
-        if user is None:
+        if not user:
             raise AuthException("missing login parameter", 401)
         return user
 
@@ -626,7 +660,7 @@ class FlaskSimpleAuth:
         if not request.is_secure:
             log.warning("password authentication over an insecure request")
         ref = self._get_user_pass(user)
-        if ref is None:
+        if not ref:
             log.debug(f"AUTH (password): no such user ({user})")
             raise AuthException(f"no such user: {user}", 401)
         if not self.check_password(pwd, ref):
@@ -639,7 +673,7 @@ class FlaskSimpleAuth:
     #
     def _get_httpauth(self):
         """Delegate user authentication to HTTPAuth."""
-        assert self._http_auth is not None
+        assert self._http_auth
         auth = self._http_auth.get_auth()
         # log.debug(f"auth = {auth}")
         password = self._http_auth.get_auth_password(auth) \
@@ -665,7 +699,7 @@ class FlaskSimpleAuth:
         assert request.remote_user is None
         auth = request.headers.get("Authorization", None)
         log.debug(f"auth: {auth}")
-        if auth is None:
+        if not auth:
             log.debug("AUTH (basic): missing authorization header")
             raise AuthException("missing authorization header", 401)
         if auth[:6] != "Basic ":
@@ -690,16 +724,19 @@ class FlaskSimpleAuth:
     def _get_param_auth(self):
         """Get user with parameter authentication."""
         assert request.remote_user is None
-        params = request.json or request.values
+        params = self._params()
         user = params.get(self._userp, None)
-        if user is None:
+        if not user:
             raise AuthException(f"missing login parameter: {self._userp}", 401)
         pwd = params.get(self._passp, None)
-        if pwd is None:
+        if not pwd:
             raise AuthException(f"missing password parameter: {self._passp}", 401)
         self._check_password(user, pwd)
         return user
 
+    #
+    # HTTP BASIC OR PARAM AUTH
+    #
     def _get_password_auth(self):
         """Get user from basic or param authentication."""
         try:
@@ -746,8 +783,7 @@ class FlaskSimpleAuth:
         p = re.match(r"^(\d{4})(\d\d)(\d\d)(\d\d)(\d\d)(\d\d)$", ts)
         if not p:
             raise Exception(f"unexpected timestamp format: {ts}")
-        iso = f"{p[1]}-{p[2]}-{p[3]}T{p[4]}:{p[5]}:{p[6]}"
-        return dt.datetime.fromisoformat(iso)
+        return dt.datetime(*[int(p[i]) for i in range(1, 7)])
 
     def _get_fsa_token(self, realm, user, delay, secret):
         """Compute a signed token for "user" valid for "delay" minutes."""
@@ -771,12 +807,12 @@ class FlaskSimpleAuth:
 
     def _can_create_token(self):
         """Whether it is possible to create a token."""
-        return self._token is not None and not \
-            (self._token == "jwt" and self._algo[0] in ("R", "E", "P") and self._sign is None)
+        return self._token and not \
+            (self._token == "jwt" and self._algo[0] in ("R", "E", "P") and not self._sign)
 
     def create_token(self, user: str = None):
         """Create a new token for user depending on the configuration."""
-        assert self._token is not None
+        assert self._token
         user = user or self.get_user()
         realm, delay = self._realm, self._delay
         if self._token == "fsa":
@@ -831,7 +867,7 @@ class FlaskSimpleAuth:
 
     def _get_any_token_auth_exp(self, token):
         """return validated user and expiration."""
-        if token is None or token == "":
+        if not token:
             raise AuthException("missing token", 401)
         return \
             self._get_fsa_token_auth(token) if self._token == "fsa" else \
@@ -850,11 +886,11 @@ class FlaskSimpleAuth:
     def _get_token_auth(self) -> Optional[str]:
         """Get authentication from token."""
         user = None
-        if self._token is not None:
+        if self._token:
             token: Optional[str] = None
             if self._carrier == "bearer":
                 auth = request.headers.get("Authorization", None)
-                if auth is not None:
+                if auth:
                     slen = len(self._name) + 1
                     if auth[:slen] == f"{self._name} ":  # FIXME lower case?
                         token = auth[slen:]
@@ -863,10 +899,10 @@ class FlaskSimpleAuth:
                 token = request.cookies[self._name] \
                     if self._name in request.cookies else None
             elif self._carrier == "param":
-                params = request.json or request.values
+                params = self._params()
                 token = params.get(self._name, None)
             else:
-                assert self._carrier == "header" and self._name is not None
+                assert self._carrier == "header" and self._name
                 token = request.headers.get(self._name, None)
             user = self._get_any_token_auth(token)
         return user
@@ -892,7 +928,7 @@ class FlaskSimpleAuth:
 
         # _user is reset before/after requests
         # so relying on in-request persistance is safe
-        if self._user is not None:
+        if self._user:
             return self._user
 
         assert self._initialized, "FlaskSimpleAuth must be initialized"
@@ -902,14 +938,14 @@ class FlaskSimpleAuth:
         for a in self._auth:
             try:
                 self._user = self._FSA_AUTH[a](self)
-                if self._user is not None:
+                if self._user:
                     break
             except AuthException as e:
                 lae = e
             # FIXME other exceptions?
 
         # rethrow last auth exception on failure
-        if self._user is None:
+        if not self._user:
             raise lae or AuthException("missing authentication", 401)
 
         log.info(f"get_user({self._auth}): {self._user}")
@@ -932,7 +968,7 @@ class FlaskSimpleAuth:
         """Clear internal caches."""
         for name in self._CACHABLE:
             fun = getattr(self, name)
-            if fun is not None and hasattr(fun, "cache_clear"):
+            if fun and hasattr(fun, "cache_clear"):
                 fun.cache_clear()
 
     #
@@ -947,10 +983,10 @@ class FlaskSimpleAuth:
 
         if ANY not in groups and ALL not in groups and \
            NONE not in groups and None not in groups:
-            assert self._user_in_group is not None, \
+            assert self._user_in_group, \
                 "user_in_group callback needed for authorize"
 
-        if auth is not None:
+        if auth:
             if isinstance(auth, str):
                 auth = [auth]
             for a in auth:
@@ -969,24 +1005,22 @@ class FlaskSimpleAuth:
                 if ANY in groups:
                     return fun(*args, **kwargs)
                 # get user if needed
-                if self._user is None:
+                if not self._user:
                     # no current user, try to get one?
                     if self._mode != "always":
                         # possibly overwrite the authentication scheme
+                        # it will be restored in an after request hook
                         # NOTE this may or may not work because other settings may
                         #   not be compatible with the provided scheme…
-                        if auth is not None:
-                            saved, self._auth = self._auth, auth
+                        if auth:
+                            self._saved_auth, self._auth = self._auth, auth
                         try:
                             self._user = self.get_user()
                         except AuthException as ae:
                             return self._Resp(ae.message, ae.status)
-                        finally:
-                            if auth is not None:
-                                self._auth = saved
                     else:
                         return self._Resp("", 401)
-                if self._user is None:  # pragma: no cover
+                if not self._user:  # pragma: no cover
                     return self._Resp("", 401)  # should be unreachable
                 # shortcut for authenticated users
                 if ALL in groups:
@@ -1045,7 +1079,8 @@ class FlaskSimpleAuth:
                     return self._Resp("missing authorization check", 500)
 
                 # translate request parameters to named function parameters
-                params = request.json or request.values
+                params = self._params()
+
                 for p, typing in typings.items():
                     # guess which function parameters are request parameters
                     if p not in kwargs:
@@ -1055,7 +1090,7 @@ class FlaskSimpleAuth:
                             except Exception as e:
                                 return self._Resp(f"type error on HTTP parameter \"{p}\" ({e})", 400)
                         else:
-                            if required is None:
+                            if not required:
                                 if p in defaults:
                                     kwargs[p] = defaults[p]
                                 else:
@@ -1121,7 +1156,7 @@ class FlaskSimpleAuth:
                         splits[i] = f"string:{spec}>{remainder}"
         newpath = "<".join(splits)
 
-        assert self._app is not None
+        assert self._app
         par = self._parameters(required=required, allparams=allparams)(view_func)
         aut = self._authorize(*roles, auth=auth)(par)
         return flask.Flask.add_url_rule(self._app, newpath, endpoint=endpoint, view_func=aut, **options)
@@ -1135,7 +1170,24 @@ class FlaskSimpleAuth:
             self.add_url_rule(rule, view_func=fun, **options)
         return decorate
 
-    # duck-typing blueprint code stealing: needs blueprints, _blueprint_order, debug
+    # support Flask 2.0 per-method decorator shortcuts
+    # note that app.get("/", methods=["POST"], ...) would do a POST.
+    def get(self, rule, **options):
+        return self.route(rule, methods=["GET"], **options)
+
+    def post(self, rule, **options):
+        return self.route(rule, methods=["POST"], **options)
+
+    def put(self, rule, **options):
+        return self.route(rule, methods=["PUT"], **options)
+
+    def delete(self, rule, **options):
+        return self.route(rule, methods=["DELETE"], **options)
+
+    def patch(self, rule, **options):
+        return self.route(rule, methods=["PATCH"], **options)
+
+    # duck-typing blueprint code stealing: needs blueprints and some other attributes.
     def register_blueprint(self, blueprint, **options):
         """Register a blueprint."""
         flask.Flask.register_blueprint(self, blueprint, **options)
